﻿using ELTE.TravelAgency.Model;
using System;
using System.Threading.Tasks;
using ELTE.TravelAgency.Data;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using NLog;

namespace ELTE.TravelAgency.Service.Controllers
{
	/// <summary>
	/// Felhasználókezelést biztosító vezérlő.
	/// </summary>
	[Route("api/[controller]")]
    [ApiController]
    [ApiConventionType(typeof(DefaultApiConventions))]
    public class AccountController : Controller
    {
        /// <summary>
        /// Authentikációs szolgáltatás.
        /// </summary>
        private readonly SignInManager<Guest> _signInManager;

        /// <summary>
        /// Naplózó szolgáltatás.
        /// </summary>
        private readonly Logger _log;

        /// <summary>
        /// Vezérlő példányosítása.
        /// </summary>
        public AccountController(SignInManager<Guest> signInManager)
        {
	        _signInManager = signInManager;
            _log = LogManager.GetLogger("account");
        }

        /// <summary>
        /// Bejelentkezés.
        /// </summary>
        /// <param name="loginDTO">Bejelentkezési adatok.</param>
        [HttpPost("login")]
        public async Task<IActionResult> Login([FromBody] LoginDTO loginDTO)
        {
            try
            {
                // bejelentkeztetjük a felhasználót
                var result = await _signInManager.PasswordSignInAsync(loginDTO.UserName, loginDTO.UserPassword, false, false);
                if (!result.Succeeded) // ha nem sikerült, akkor nincs bejelentkeztetés
                {
                    _log.Info("LOGIN FAILED for user {0} from address {1}", loginDTO.UserName, HttpContext.Connection.RemoteIpAddress);
                    return Forbid();
                }

                // ha sikeres volt az ellenőrzés
                _log.Info("LOGIN SUCCESS for user {0} from address {1}", loginDTO.UserName, HttpContext.Connection.RemoteIpAddress);
                return Ok();
            }
            catch
            {
				// Internal Server Error
	            return StatusCode(StatusCodes.Status500InternalServerError);
			}
        }

        /// <summary>
        /// Kijelentkezés.
        /// </summary>
        [HttpGet("logout")]
        [Authorize] // csak bejelentkezett felhasználóknak
        public async Task<IActionResult> Logout()
        {
            try
            {
				// kijelentkeztetjük az aktuális felhasználót
				await _signInManager.SignOutAsync();
				return Ok();
            }
            catch
            {
				// Internal Server Error
	            return StatusCode(StatusCodes.Status500InternalServerError);
			}
        }
    }
}
